<?php
/* $Id: helper.php 68 2010-06-12 16:54:21Z mir $ */
include_once 'config.inc.php';
require_once 'persistens.php';

define('MICRO', 3);
define('MINOR', 7);
define('MAJOR', 0);

$VERSION = MAJOR.".".MINOR.".".MICRO;
$CURRENT_VERSION = $VERSION;
$PREFIX = TOP_FOLDER."/install/".DBDRIVER;
$PATCH_SET = array(
    "173" => "db_upgrade_0_7_3.sql"
);

$pwd = WEB_ROOT;
if ($pwd[strlen($pwd)-1] == '/')
    $pwd = substr($pwd, 0, -1);

function string2int($str) {
    if (is_numeric($str))
        return $str;
    if (is_string($str)) {
        $parts = explode(".", $str);
        $str = 0;
        for ($i = 0; $i < count($parts); $i++) {
            $num = ($i == 0) ? $parts[$i] + 1 : $parts[$i];
            $str += (pow(10, count($parts) - $i - 1) * $num);
        }
    }
    return $str;
}

function getServerUrl() {
    $protocol = 'http://';
    $server = ($_SERVER['SERVER_PORT'] != 80)? $_SERVER['SERVER_NAME'].':'.$_SERVER['SERVER_PORT'] : $_SERVER['SERVER_NAME'];
    return $protocol.$server.WEB_ROOT;
}

function getServerUri() {
    return TOP_FOLDER;
}

function createKey($key) {
    $td = mcrypt_module_open('rijndael-256', '', 'ofb', '');
    $public = mcrypt_create_iv(mcrypt_enc_get_iv_size($td), MCRYPT_DEV_RANDOM);
    $ks = mcrypt_enc_get_key_size($td);
    mcrypt_module_close($td);

    /* Create key */
    $secret = substr($key, 0, $ks);
    return array($secret, $public);
}

function encode($text) {
    /* Intialize encryption */
    $td = mcrypt_module_open('rijndael-256', '', 'ofb', '');
    $iv = $_SESSION['authenticate']->getPublicKey();
    $key = $_SESSION['authenticate']->getSecretKey();

    mcrypt_generic_init($td, $key, $iv);

    /* Encrypt data */
    $encrypted = mcrypt_generic($td, $text);

    /* Terminate encryption handler */
    mcrypt_generic_deinit($td);
    mcrypt_module_close($td);

    return $encrypted;
}

function decode($text) {
    $td = mcrypt_module_open('rijndael-256', '', 'ofb', '');
    $iv = $_SESSION['authenticate']->getPublicKey();
    $key = $_SESSION['authenticate']->getSecretKey();

    /* Initialize encryption module for decryption */
    mcrypt_generic_init($td, $key, $iv);

    /* Decrypt encrypted string */
    $ciffer = quoted_printable_decode($text);
    $decrypted = mdecrypt_generic($td, $ciffer);

    /* Terminate decryption handle and close module */
    mcrypt_generic_deinit($td);
    mcrypt_module_close($td);

    return $decrypted;
}

function implode_cal($text) {
    $infos = explode("\n", $text);
    foreach ($infos as $info) {
        $row = explode('=', $info);
        $result[$row[0]] = $row[1];
    }
    return $result;
}


// taken from PHP documentation comments
function webcal_quoted_printable_encode($input, $line_max = 76) {
	$hex = array('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
	$lines = preg_split("/(?:\r\n|\r|\n)/", $input);
	$eol = "\r\n";
    $linebreak = '=0D=0A';
    $escape = '=';
    $output = '';

    for ($j=0; $j<count($lines); $j++) {
        $line = $lines[$j];
        $linlen = strlen($line);
        $newline = '';
        for($i = 0; $i < $linlen; $i++) {
            $c = substr($line, $i, 1);
            $dec = ord($c);
            if (($dec == 32) && ($i == ($linlen - 1))) {
				// convert space at eol only
                $c = '=20';
            }
            elseif (($dec == 61) || ($dec < 32 ) || ($dec > 126)) {
				// always encode "\t", which is *not* required
                $h2 = floor($dec/16);
                $h1 = floor($dec%16);
                $c = $escape.$hex["$h2"] . $hex["$h1"];
            }
            if ((strlen($newline) + strlen($c)) >= $line_max) {
				// CRLF is not counted
                $output .= $newline.$escape.$eol;
                // soft line break; " =\r\n" is okay
                $newline = "    ";
            }
            $newline .= $c;
        }
        // end of for
        $output .= $newline;
        if ($j < count($lines) - 1) {
            $output .= $linebreak;
        }
    }
    return trim($output);
}

function newUpdates() {
    global $VERSION, $CURRENT_VERSION;
    
    $con = Persistens::getInstance(DBDRIVER);
    try {
        $res = $con->getVersion();
    }
    catch (Exception $e) {
        ReportError($e);
    }
    $version = $res['version'];
    $version = string2int($version);
    if (! is_numeric($version)) {
        ReportError("Invalid version format");
        exit;
    }
    $CURRENT_VERSION = $version;
    file_put_contents('/tmp/updates', string2int($VERSION)." > $version", FILE_APPEND);
    return string2int($VERSION) > $version;
}

function upgrade() {
    global $VERSION, $PATCH_SET, $CURRENT_VERSION, $PREFIX;

    $con = Persistens::getInstance(DBDRIVER);
    try {
        $res = $con->getVersion();
    }
    catch (Exception $e) {
        ReportError($e);
        exit;
    }
    if ($CURRENT_VERSION < 173) {
		if (($res = $con->execute(
			"create table backup as select * from calendar")) !== true) {
			ReportError($res);
			exit;
		}
		$rows = $con->getCalendarConfig();
		if (! is_array($rows)) {
			ReportError($rows);
			exit;
		}
		foreach($rows as $row) {
			if (function_exists(quoted_printable_encode))
				$ciffer = quoted_printable_encode($row['config']);
			else
				$ciffer = webcal_quoted_printable_encode($row['config']);
            if (($res = $con->execute("begin transaction")) !== true) {
                ReportError($res);
                exit;
			}
			if (($res = $con->execute(
				"update calendar set config = '$ciffer' where id = " .
				$row['id'])) !== true) {
				$con->execute("rollback");
				ReportError($res."<br/>A backup of your calendar data can be found in table backup");
				exit;
			}
		}
		$con->execute("commit");
	}
    foreach ($PATCH_SET as $set => $file) {
        //echo "$set -> $file<br/>";
        if ($CURRENT_VERSION < $set) {
            //echo $PREFIX."_".$file."<br/>";
            if (! file_exists($PREFIX."_".$file))
                continue;
            $sql = file_get_contents($PREFIX."_".$file);
            $lines = explode(";", $sql);
            if (count($lines) > 0) {
                if (($res = $con->execute("begin transaction")) !== true) {
                    ReportError($res);
                    exit;
				}
                foreach ($lines as $line) {
                    if (($res = $con->execute($line)) !== true) {
                        ReportError($res);
                        $con->execute("rollback");
                        exit;
                    }
                }
                $con->execute("commit");
            }
        }
        else
            break;
    }
}

function ReportError($msg) {
    // be sure that the supplied parameter is a string and not empty
    if (empty ($msg) || !is_string ($msg)) {
        throw new ErrorException ('Invalid parameter supplied to ReportError', 0, E_ERROR);
    }

    // retrieve error settings
    $display    = strtolower(ini_get('display_errors'));
    $log        = strtolower(ini_get('log_errors'));

    // check if we're displaying errors
    if ($display === 'on' || $display === '1' || $display === 1 || $display === 'true' || $display === true) {
        echo $msg;
    }
    else {
        print '<?xml version="1.0" encoding="utf-8"?>
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
            "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
        <html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
            <head>
                <title>Web Calendar</title>
                <link rel="stylesheet" type="text/css"
                    href="'.$pwd.'/css/top_level.css" />
                <link rel="shortcut icon"
                    href="'.$pwd.'/pixmaps/favicon.ico" />
            </head>
            <body>
                <div id="error_msg">
                    <p>
                        The application has triggered an error which is not
                        recoverable. The execution has therefore been cancelled.
                        <br/><br/>
                        The error is logged and the webmaster will handle
                        the error in due time.
                        <br/><br/>
                        To help fix the application you are welcome to notify
                        the <a href="mailto:'.$_SERVER['SERVER_ADMIN'].'
                        ?subject='.TITLE.': error report['.
                        $_SERVER['SERVER_NAME'].']">webmaster</a>
                        by email with a detailed description containing the
                        following information:
                        <br/><br/>
                        1) What did you do just before this message occurred<br/>
                        2) What did you expect to happen<br/>
                        3) Your browser name and version number.<br/>
                    </p>
                </div>
            </body>
        </html>';
    }

    // check if we're logging errors
    if ($log === 'on' || $log === '1' || $log === 1 || $log === 'true' || $log === true) {
        $result = error_log ($msg);

        // check for error while logging
        if (!$result) {
            throw new ErrorException ('Attempt to write message to error log failed in ReportError', 0, E_ERROR);
        }
    }
}

//Begin main function definition
function ErrorsAsExceptions($level, $msg, $fileName, $lineNumber) {
    // do nothing if error reporting is turned off
    if (error_reporting() === 0) {
        return;
    }

    // be sure received error is supposed to be reported
    if (error_reporting() & $level) {
        try {
            // report error to appropriate channels
            ReportError(/*$reportMsg*/$msg);
        }
        catch (ErrorException $e) {
            // ignore errors while reporting
        }

        // go ahead and throw the exception
        throw new ErrorException($msg, 0, $level, $fileName, $lineNumber);
    }
}

set_error_handler('ErrorsAsExceptions');

?>
